#
# spec file for package python-traits
#
# Copyright (c) 2025 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


# cycle with pyface
%global flavor test%{nil}
%if "%{flavor}" == "test"
%define psuffix -test
%bcond_without test
%else
%define psuffix %{nil}
%bcond_with test
%endif
Name:           python-traits%{psuffix}
Version:        6.4.3
Release:        3.1
Summary:        Explicitly typed attributes for Python
# Images have different licenses. For image license breakdown check
# image_LICENSE.txt file. Except enthought/traits/ui/editors_gen.py
# which is GPLv2+ all remaining source or image files are in BSD
# 3-clause license. Confirmed from upstream.
License:        BSD-3-Clause AND EPL-1.0 AND LGPL-2.1-only
URL:            https://github.com/enthought/traits
Source:         https://github.com/enthought/traits/archive/%{version}.tar.gz#/traits-%{version}.tar.gz
Patch1:         https://github.com/enthought/traits/commit/a20f2154b2c79eb8550ea9228d1a4415ff51b72a.patch#/py313-ctraits.patch
BuildRequires:  %{python_module devel}
BuildRequires:  %{python_module pip}
BuildRequires:  %{python_module setuptools}
BuildRequires:  %{python_module wheel}
BuildRequires:  fdupes
BuildRequires:  python-rpm-macros
# SECTION test requirements
%if %{with test}
BuildRequires:  %{python_module Cython}
BuildRequires:  %{python_module pyface}
BuildRequires:  %{python_module numpy if (%python-base without python36-base)}
%endif
# /SECTION
Recommends:     python-traitsui >= 7.0.0

%python_subpackages

%description
The traits package developed by Enthought provides a special type definition
called a trait. Although they can be used as normal Python object attributes,
traits also have several additional characteristics:

 * Initialization: A trait can be assigned a default value.
 * Validation: A trait attribute's type can be explicitly declared.
 * Delegation: The value of a trait attribute can be contained either
   in another object.
 * Notification: Setting the value of a trait attribute can trigger
   notification of other parts of the program.
 * Visualization: User interfaces that permit the interactive
   modification of a trait's value can be automatically constructed
   using the trait's definition.

Part of the Enthought Tool Suite (ETS).

%prep
%autosetup -p1 -n traits-%{version}
%fdupes examples/

%build
%if %{without test}
export CFLAGS="%{optflags} -fno-strict-aliasing"
%pyproject_wheel
%endif

%install
%if %{without test}
%pyproject_install
%python_expand %fdupes %{buildroot}%{$python_sitearch}
%endif

%check
%if %{with test}
%{python_expand mkdir tester_%{$python_bin_suffix}
pushd tester_%{$python_bin_suffix}
export PYTHONPATH=%{buildroot}%{$python_sitearch}
$python -m unittest discover -v traits
popd
}
%endif

%if %{without test}
%files %{python_files}
%defattr(-,root,root,-)
%doc CHANGES.rst README.rst
%doc examples/
%license LICENSE.txt image_LICENSE*.txt
%{python_sitearch}/traits/
%{python_sitearch}/traits-%{version}.dist-info
%endif

%changelog
* Fri May  9 2025 Steve Kowalik <steven.kowalik@suse.com>
- Switch to pyproject macros.
* Tue Nov 19 2024 Dirk Müller <dmueller@suse.com>
- add py313-ctraits.patch to fix build with python 313
* Thu Feb 22 2024 Steve Kowalik <steven.kowalik@suse.com>
- Drop patch fix-test.patch, it wasn't even applied.
- Switch to autosetup macro.
* Fri Jan 12 2024 Dirk Müller <dmueller@suse.com>
- update to 6.4.3:
  * This is a bugfix release that fixes test
    failures with Sphinx 7.2 and later, and adds support for
    Python 3.12
  * Rename requires_numpy_testing decorator to
    requires_numpy_typing, and have it check for numpy.typing,
    not numpy.testing.
  * Fix missing numpy_examples directory in traits-stubs package
    data.
  * Validation of items within a container (e.g., foos =
    List(MyTraitType)) now always matches the validation used for
    the item trait at top level (e.g., foo = MyTraitType).
    Previously, the validation methods used could differ, thanks
    to a bug in the container implementations. For most trait
    types this will make no difference, but for the Tuple trait
    type this change has the consequence that lists will no
    longer be accepted as valid for Tuple traits inside list
    items. See issue #1619 and PR #1625 for more information.
  * Related to the above: a top-level Tuple() trait declaration
    currently accepts Python list objects, while a Tuple
    declaration with explicit item types (for example
    Tuple(Int(), Int())) does not. The support for list objects
    in plain Tuple() is deprecated, and will be removed in a
    future version of Traits. See PR #1627 for more information.
  * The following people contributed code changes for this
    release: * Caio Agiani * Steve Allen * Mark Dickinson * Sai
    Rahul Poruri * Corran Webster Features ~~~~~~~~ * ETSConfig
    attributes now support deletion. This makes it easier to make
  * Complex trait type validation is now more lenient: any type
    that implements __complex__ will be accepted.
  * BaseFloat validation is now more lenient, and matches Float
    validation: BaseFloat now also accepts objects whose type has
    an __index__ method.
  * An enumerate alias has been removed from traits.trait_base.
    In the unlikely event of code that imports enumerate from
    traits.trait_base, use the built-in enumerate instead.
  * Fast validation tuples int_fast_validate, float_fast_validate
    and complex_fast_validate have been removed from the
    traits.trait_types module.
  * TraitListObject, TraitDict object and TraitSetObject now use
    the validate method of the appropriate CTrait instances to
    validate items, keys and values. Previously the handler's
    validate method was used; this gave buggy behaviour in cases
    where the handler's validate method differed from the actual
    validation in use.
  * Fix specification of default_value that incorrectly
    disregarded default_value_type.
  * Fix incorrect results from  clone_traits applied to List,
    Dict and Set traits.
  * The find_resource and store_resource tests are now skipped if
    the pkg_resources module is not present in the environment.
  * An ETSConfig test has been renamed so that it's properly
    picked up by the test runner.
  * Fix some ETSConfig tests that assume unittest as the test
    runner.
  * Rename various test-related classes to avoid pytest trying to
    harvest test methods from them.
  * Overriding a default for a List or other collection trait in
    a subclass now works as expected. Previously, the behaviour
    was unusably buggy.
  * Tuple traits currently accept Python list objects in some
    (but not all) circumstances. That feature is deprecated, and
    will be removed in a future version of Traits.
  * Add stubs for Array, ArrayOrNone, and CArray.
  * Fix various stubs for traits.trait_types; add stubs for
    traits.ctraits.
  * Fix that TraitError stubs weren't exposed at traits.api
    level.
  * Make Int and Float type stubs more accurate.
  * Fix incorrect type stubs for the Dict trait type.
  * Make ETSConfig class documentation visible in the API docs.
  * Add copy buttons to code samples in documentation.
  * Document Date, Datetime and Time trait types.
  * Fix some missing mentions of Set in notification docs.
  * Document the 'some_trait.-' pattern for on_trait_change.
  * Document that Either should not be used in new code.
  * Document that TraitPrefixMap and TraitPrefixList are
    deprecated.
  * Document that the Trait factory function should not be used
    in new code.
  * Miscellaneous minor fixes.
  * Don't collect traits-stubs tests under pytest. As a result of
    this and other fixes, the test suite now passes under pytest.
  * Update etstool.py for Python 3.8 support. Python 3.8 is now
    the default Python version for builds.
  * Use PySide6 for Python >= 3.8 instead of PySide2 in CI
    testing.
* Wed Oct 27 2021 Steve Kowalik <steven.kowalik@suse.com>
- Update to 6.3.1:
  * Make PrefixMap._map available again, for compatibility with Mayavi. (#1578)
  * Support for Python 3.10 has been added.
  * The observe mini-language now supports use of "*" for listening to all
    traits on a HasTraits object. Currently this support is limited to
    cases where the "*" appears in a terminal position. For example,
    observe("foo:*") is supported, but observe("*:foo") is not. (#1496, #1525)
  * The Any trait type now supports a factory argument (with accompanying
    args and kw arguments). This can be used to specify a per-instance
    default, for example with Any(factory=dict). (#1557, #1558)
  * When a method is decorated with an observe decorator, the method
    signature is now checked, and a warning issued if it doesn't match the
    expected signature.  (#1529)
  * The Date, Datetime and Time trait types have a new argument allow_none.
    (#1432)
  * The Date trait type has a new argument allow_datetime .(#1429)
  * The ObserverGraph instances that result from compiling ObserverExpression
    objects and observe mini-language strings are now cached.
  * The equality definition on ObserverExpression has been simplified. (#1517)
  * A failure to parse an observe mini-language string now raises ValueError
    rather than LarkError. (#1507)
  * Invalid assignments to PrefixList and PrefixMap traits produced an
    unnecessarily nested exception. This has been fixed. (#1564)
  * An observe-decorated listener method whose name has the special form
    "_traitname_changed" will no longer be triggered both as as result of the
    observe decorator and the special naming: it will only be triggered via
    the observe decorator. (#1560)
  * The delegate parameter was mistyped in the typing stubs for the Delegate
    trait type. This has been fixed. (#1556)
  * The Function and Method trait types will no longer fail when arguments are
    passed. Note that these trait types are already deprecated, and should not
    be used in new code. (#1543)
  * A use of the (deprecated) distutils library has been replaced with
    sysconfig. (#1452)
  * Dynamic default handing has been fixed in the _instance_handler_factory
    used by the TraitsUI TableEditor. (#1446, #1450)
  * A bad observe-decorated listener signature in a test has been fixed. (#1530)
- Remove nose from BuildRequires, the nose portion of the testsuite is
  firmly deprecated.
* Tue Feb 23 2021 Ben Greiner <code@bnavigator.de>
- unskip python36. NumPy is actually optional.
* Sun Feb 21 2021 andy great <andythe_great@pm.me>
- Skip python 3.6.
- Change source URL to Github.
- Update to version 6.2.0.
  * The Traits examples are now distributed as part of the Traits
    egg, and are contributed to the etsdemo application.
  * Performance of the observe framework has been significantly
    improved.
  * It's no longer necessary to specify a trait comparison mode of
    ComparisonMode.identity when using observe to observe items
    in a List, Dict or Set.
  * When importing from Traits, you should always import from one
    of the api modules (for example, traits.api,
    traits.adaptation.api, etc.) This recommendation has now been
    made explicit in the documentation. If you find something you
    need that's not available from one of the api modules, please
    let the Traits developers know.
* Mon Jul 13 2020 Marketa Calabkova <mcalabkova@suse.com>
- Update to 6.1.0
  * A new :mod:`observation <traits.observation>` framework for observing traited
    attributes and other observable objects has been introduced. This is intended
    to provide a full replacement for the existing :func:`on_trait_change`
    mechanism, and aims to fix a number of fundamental flaws and limitations of
    that mechanism. See the :ref:`observe-notification` section of
    the user manual for an introduction to this framework.
  * New :class:`~traits.trait_list_object.TraitList`,
    :class:`~traits.trait_dict_object.TraitDict` and
    :class:`~traits.trait_set_object.TraitSet` classes have been added,
    subclassing Python's built-in :class:`python:list`, :class:`python:dict` and
    :class:`python:set` (respectively). Instances of these classes are observable
    objects in their own right, and it's possible to attach observers to them
    directly. These classes were primarily introduced to support the new
    observation framework, and are not expected to be used directly. The API for
    these objects and their notification system is provisional, and may change in
    a future Traits release.
  * A new :class:`.Union` trait type has been added. This is intended as a
    simpler replacement for the existing :class:`.Either` trait type, which
    will eventually be deprecated.
  * New :class:`.PrefixList`, :class:`.PrefixMap` and :class:`.Map` trait types
    have been added. These replace the existing :class:`.TraitPrefixList`,
    :class:`.TraitPrefixMap` and :class:`.TraitMap` subclasses of
    :class:`.TraitHandler`, which are deprecated.
  * Typing stubs for the Traits library have been added in a
    ``traits-stubs`` package, which will be released separately to PyPI. This
    should help support Traits-using projects that want to make use of type
    annotations and type checkers like `mypy <http://mypy-lang.org/>`_.
  * Python 2.7 is no longer supported; Traits 6.0 requires Python 3.5 or later.
  * Trait types related to Python 2 (for example ``Unicode`` and ``Long``) have
    been deprecated in favour of their Python 3 equivalents (for example ``Str``
    and ``Int``).
  * Many little-used historical features of Traits have been deprecated, and
    are scheduled for removal in Traits 7.0.
  * Some historical features of Traits that had no evidence of external usage
    were removed in Traits 6.0.
  * Introspection of ``CTrait`` and ``HasTraits`` objects is greatly improved.
    All of the internal state that was previously hidden within the C extension
    is now accessible from Python.
  * The Traits codebase has undergone some significant reorganizations,
    reformattings and style cleanups to make it easier to work with, and
    to improve the separation between Traits and TraitsUI.
  * This release was focused mainly on cleanup and bugfixing. Nevertheless,
    it contains a sprinkling of new features. There's a new ``Datetime``
    trait type. The ``Enum`` trait type now supports Python enumerations.
    The ``File`` trait type supports path-like objects.
  * And many more... see upstream changelog
- Convert to multibuild to break build cycle with pyface
- Add patch fix-test.patch to make the tests pass
* Thu Jan 30 2020 Todd R <toddrme2178@gmail.com>
- Update to 5.2.0:
  + Enhancements
  * Support installation from source archives.
  + Fixes
  * Ensure ``TraitListEvent.index`` is always an integer.
  * Update the deprecated ``collections.MutableMapping`` import.
  * Fix inadvertent modification of the ``Category`` base class.
  * Rework version handling in ``setup.py``.
  * Don't autogenerate documentation for ``ViewElement``.
  * Ensure that all tests are ``unittest`` compatible.
  + Changes
  * Replace occurences of deprecated ``AdaptsTo`` with ``Supports``.
  * Remove ``Class`` trait.
  * Deprecate ``Category`` trait.
  * Fix typos in docstrings.
  * Use decorator form of ``classmethod``.
  * Remove redefinition of ``NullHandler``.
  * Add an import check helper.
  * Clean up Cython tests.
  * Clean up test output.
  + Miscellaneous
  * Update EDM version on CI to version 2.0.0.
  * Don't finish fast on CI.
  * Use ``unittest`` to run tests in CI.
  * Low-level fixes and style cleanup in ``etstool.py``.
  * Add ``--editable`` option for ``install``, ``update`` CI commands.
  * Make git commit hash available to archives.
  * Fix use of non-edm envs as bootstrap envs on Windows.
  * Remove edm installed package before installing from source.
  * Add help text to click options.
  * Various cleanups, fixes and enhancements in ``etstool.py``.
* Mon Jul 22 2019 Tomáš Chvátal <tchvatal@suse.com>
- Update to 5.1.2:
  * Traits documenter no longer generates bad reST for traits whose definition spans multiple source lines. (#494)
* Tue Apr 23 2019 pgajdos@suse.com
- version update to 5.1.1
  * Revert a change (#449) which accidentally broke external uses of
    ``_py2to3.str_find`` and ``_py2to3.str_rfind``. (#472)
* Wed Apr 17 2019 pgajdos@suse.com
- version update to 5.1.0
  * Make UUID trait initializable. (#459)
  * Change default ``FileEditor`` behavior for a ``File`` trait based on
    whether ``exists=True`` is specified for that trait. (#451, #467)
  * The changes made in #373 to make dynamically-added traits pickleable have
    been reverted. (#462)
  * ``traits.api.python_version`` has been removed. Internals have been
    refactored to use ``six.PY2`` in preference to ``sys.version_info``.
    (#449)
  * Don't depend on the 3rd party ``mock`` library on Python 3; use
    ``unittest.mock`` instead. (#446)
- Drop no longer needed avoid_sys_modules_hackery.patch
* Thu Feb 21 2019 Todd R <toddrme2178@gmail.com>
- Add avoid_sys_modules_hackery.patch
  Fixes building with latest numpy
  see: gh#enthought/traits#441
* Wed Feb  6 2019 Todd R <toddrme2178@gmail.com>
- Update to 5.0.0
  + highlights of this release are
  * Removal of 2to3 fixers and the use of six to provide Python 2/3 compatibility
  * Removal of deprecated `traits.protocols` submodule and related utils.
  * New `HasRequiredTraits` class
  * Better IPython tab completion for `HasTraits` subclasses
* Wed May  9 2018 toddrme2178@gmail.com
- Use license tag
* Wed Aug 23 2017 toddrme2178@gmail.com
- Implement single-spec version
- Update to version 4.6.0
  Highlights of this release include:
  * support for Python 3.4 and 3.5.
  * new Bytes and ValidatedTuple traits.
  * a new ArrayOrNone trait which correctly handles None comparisons with Numpy
    arrays.
  * clean-up of the ETSConfig code for TraitsUI toolkit selection.
  * better compatibility with NumPy scalar types.
  * many other bugfixes and improvements.
* Mon Oct 13 2014 toddrme2178@gmail.com
- Updated to version 4.5.0
  + Traits is now compatible with Python 3!
    The library now supports Python 3.2 and 3.3 .
  + The release also includes increased code coverage and automatic
    coverage report through coveralls.io .
  + Enhancements
  * Test files cleanups (#108, #111, #121)
  * Add automatic coverage reports (#110, #122)
  * Removed obsolete code (#109, #112, #113)
  * Increased test coverage (#114, #118)
  * Python 3 support (#115).  Thanks Yves Delley.
  * Allow setting and resetting the global adaptation manager (#145)
  * Various documentation improvements (#132, #133, #148, #154).
  + Changes
  * The Int trait type now accepts Python ints *and* Python longs, as well as
    instances of any Python type that implements the `__index__` method.
    Previously, long instances were not accepted. (#104, #123).
  + Fixes
  * Fix crash when trying to validate a property that has been deleted. (#138)
  * Fix clearing exception when raising a TraitError (#119)
  * Fix automatic adaptation when assigning to List trait (#147)
  * Fix some ctraits refcounting and exception clearing bugs (#48).  Thanks Yves
    Delley.
- Updated to version 4.4.0
  + The major new feature in this release is a new adaptation mechanism in the
    ``traits.adaptation`` package.  The new mechanism is intended to replace the
    older traits.protocols package.  Code written against ``traits.protocols`` will
    continue to work, although the ``traits.protocols`` API has been deprecated,
    and a warning will be logged on first use of ``traits.protocols``.  See the
    'Advanced Topics' section of the user manual for more details.
  + The release also includes improved support for using Cython with `HasTraits`
    classes, some new helper utilities for writing unit tests for Traits events,
    and a variety of bug fixes, stability enhancements, and internal code
    improvements.
  + New features
  * The adaptation mechanism in Traits, formerly based on the 'traits.protocols'
    package, has been replaced with the more robust 'traits.adaptation'
    package. (#51)
  * Added utility function for importing symbols (name, classes, functions)
    by name: 'traits.util.api.import_symbol'. (#51)
  * Users can set a global tracer, which receives all traits change events:
    `traits.trait_notifiers.set_change_event_tracers`. (#79)
  + Enhancements
  * Update benchmark script. (#54)
  * traits.util.deprecated: use module logger instead of root logger. (#59)
  * Provide an informative message in AdaptationError. (#62)
  * Allow HasTraits classes to be cythonized. (#73)
  * Improve tests for cythonization support. (#75)
  * Extending various trait testing helpers (#53)
  + Refactoring
  * The Traits notification code has been reworked to remove code duplication,
    and test coverage of that code has been significantly improved. (#79)
  + Fixes
  * Fix race condition when removing a traits listener. (#57)
  * Fix ugly interaction between DelegatesTo change handlers, dynamic change
    handlers and two levels of dynamic intialization. (#63)
  * Use a NullHandler for all 'traits' loggers. (#64)
  * Fix race condition in TraitChangeNotifyWrapper.listener_deleted (#66)
  * Fix leaking notifiers. (#68)
  * Fix failing special instance trait events. (#78)
  * Fix hiding KeyError exception inside trait default initialize method.
    (#81)
  * Fix Adapter object initialization. (#93)
  * Fix cyclic garbage arising from use of the WeakRef trait type. (#95)
  * `TraitSetObject.copy` now returns a plain rather than an
    uninitialized `TraitSetObject` instance. (#97)
  * Fix cyclic garbage arising from dynamic trait change handlers. (#101)
- Updated source to pypi url.
* Thu Oct 24 2013 speilicke@suse.com
- Require python-setuptools instead of distribute (upstreams merged)
* Sat Apr 20 2013 toddrme2178@gmail.com
- Added a note about being part of the Enthought Tool Suite (ETS).
* Thu Apr 18 2013 toddrme2178@gmail.com
- Update to 4.3.0
  * no changelog
* Tue Jun 26 2012 toddrme2178@gmail.com
- Fix building on SLE 11
* Tue May 22 2012 toddrme2178@gmail.com
- changed name to python-traits to match upstream naming
- updated to version 4.2.0
- updated to version 4.1.0
* Sat Jun 25 2011 ocefpaf@yahoo.com.br
- updated to version 4.0.0
- The 'enthought' namespace has been removed For example:
  from enthought.traits.api import HasTraits
  is now simply:
  from traits.api import HasTraits
* Tue May 31 2011 saschpe@suse.de
- Spec file cleanup:
  * Removed unused script code
  * README.txt and LICENSE.txt belong into the base package
  * Proper SUSE version checks for fdupes, fixes SLE build
  * Added license header
* Sun Mar 20 2011 ocefpaf@yahoo.com.br
- first import from Fedora src rpm
